/**
* 	@file BubbleChart.java
*	@author Sam Lucas
* 	@date 04 Apr 2013
* 
*
*	@brief Stores a user defined BubbleChart to be displayed on screen
*	@see Chart
*	Provides an interface for designing and displaying a BubbleChart
*	Storing internal chart data and providing visual functions.
* 
*/



import org.jfree.chart.ChartFactory;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.data.general.Dataset;
import org.jfree.chart.renderer.xy.XYItemRenderer;
import org.jfree.data.xy.DefaultXYZDataset;
import  java.util.ArrayList;
import  java.awt.Color;
import org.jfree.data.xy.XYZDataset;



import	javax.swing.JFrame;

public class BubbleChart extends Chart {
	
	/**
	* Construct a BubbleChart
	*
	* @param data is the data that is used to create the chart
	* @param title is the title of the chart
	* @param colourScheme is a string of the type of colour that will define 
	* the colour of the bubbles
	* @param userList is the ArrayList<Color> of user defined colours
	* 
	*/
	public BubbleChart(ChartData data, String title, String colourScheme, 
													ArrayList<Color> userList){
		super(data, title, colourScheme, userList);
		SetXAxis(data.GetColumn(0));
		SetYAxis(data.GetColumn(1));
		SetLegend(GetChart().getLegend());
		SetDate(true);
	}

	/**
	* Create a chart of type BubbleChart. 
	*
	* @param plotData is the data that shall be used to create the chart
	* @param title is the title to be used upon creation of the chart
	*
	* @return the newly created BubbleChart as chart 
	*/
	public JFreeChart CreateChart(Dataset plotData, String title) {
	    JFreeChart chart = ChartFactory.createBubbleChart(
		title,							//	Chart title
		GetData().GetColumn(0),			// 	Domain/x axis label
		GetData().GetColumn(1), 		// 	Range/y axis label 
		(DefaultXYZDataset) plotData,	//	Data
		PlotOrientation.VERTICAL,		//	Orientation
		true,					//	Include legend
		true,					//	Include tooltips
		false);					//	Do not include URLs

	    return chart;
	}
		
	/**
	* Change the currently selected chart to have a colourScheme 
	* as selected by the user 
	*
	* @param colourScheme is the Colour Scheme which has been selected by the user
	* which is to be mapped onto the charts data.
	*
	* @return boolean indicating True if colourScheme was successfully implemented
	* or false otherwise.
	*/
	boolean ApplyColour(ColourList colourScheme) {
		boolean	success = false;
		XYPlot plot = (XYPlot) GetChart().getPlot();
		if (plot != null) {
			for (int i = 0; i < GetData().Width(); i++) {
				plot.getRenderer().setSeriesPaint(i, GetColour().Get(i));
			}
			success = true;
		}
		return success;
	}

	/**
	* Create a dataset of type DefaultXYZDataset using data of type ChartData 
	*
	* @param data is the data that shall be used to create the chart
	*
	* @return the newly created dataset as result 
	*/
	Dataset CreateDataset(ChartData data) { 
		DefaultXYZDataset result = new DefaultXYZDataset();
		double[][] dataarray = new double[data.Height()][data.Width()];
		for (int col = 0; col < data.Height()	; col++) {
			for (int row = 0; row < data.Width()	; row++) {
			dataarray[col][row] = (Double.parseDouble(data.Get(col, row)));
			}
		}
		result.addSeries(data.GetColumn(2), dataarray);
		
		return result;
		
	}
	
	/**
	*	Unit test
	*	@param args - command line arguments (not used)
	*/
	public static void main(String[] args) {
		final int			WINDOW_SIZE = 700;
		final String[][]	TEST_DATA = {{"5","14","3","25","32"},
										{"23","21","6","5","11"},
										{"3","1","6","5","2"}};
		final String[]		TEST_COLUMNS = {"Test","Data", "Yeah"};
		JFrame		frame = new JFrame("BubbleChart Test");
		ChartData 	data = new ChartData(TEST_COLUMNS, TEST_DATA);
		BubbleChart	chart = new BubbleChart(data,
										"Bubble Test", ColourGenerator.RANDOM,null);
		System.out.println(chart);
		frame.add(chart);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setSize(WINDOW_SIZE, WINDOW_SIZE);
		frame.setVisible(true);		
	}
	
	
} 