/**	
*	@file	Chart.java
*	@author	Jason Enderby
*	@date	06 Apr 2013
*	@see	ColourList, ChartData, ColourGenerator, Visualisation
*	
*	@brief Stores how chart should be displayed from its respective data
*	
*	Provides an interface for designing and displaying charts in a JSwing
*	enviroment. Storing internal chart data and providing visual functions.
*	
*/



import	org.jfree.chart.JFreeChart;
import	org.jfree.data.general.Dataset;
import	org.jfree.chart.ChartPanel;
import  org.jfree.chart.plot.CategoryPlot;
import  org.jfree.chart.plot.XYPlot;
import  org.jfree.ui.RectangleEdge;
import	java.util.ArrayList;
import	java.awt.Color;
import  org.jfree.ui.HorizontalAlignment;
import  org.jfree.chart.title.TextTitle;
import  org.jfree.chart.title.DateTitle;
import  org.jfree.chart.title.LegendTitle;
import  java.text.DateFormat;





public abstract class Chart extends Visualisation {
	protected JFreeChart	m_Chart;
	protected String 		m_ChartTitle;
	protected String		m_Author = "";
	protected Boolean		m_Date = false;
	protected String		m_Description = "";
	protected String		m_XAxis = "";
	protected String		m_YAxis = "";
	protected LegendTitle 	m_Legend;
	
	
	/**
	*	Set the Title of the chart.
	*
	*	@param title - New title to be added
	*
	*	@return - success
	*/
	public boolean SetChartTitle(String title) {
		boolean	success = true;
		if (title != null) {
			success = true;
			m_ChartTitle = title;
		}
		updateTitles();
		return success;
	}
	
	/**
	*	Accessor for m_ChartTitle
	*
	*	@return - Chart Title String
	*/
	public String GetChartTitle() {
		return m_ChartTitle;
	}
	
	/**
	*	Set the Author of the chart. 
	*
	*	@param author is the String that the user has specified to be used
	*
	*	@return - success
	*/
	public boolean SetAuthor(String author) {
		boolean success = false;
		if (author != null) {
			success = true;
			m_Author = author;
		}
		updateTitles();
		return success;
	}
	
	/**
	*	Accessor for m_Author
	*
	*	@return - Author String
	*/
	public String GetAuthor() {
		return m_Author;
	}
	
	/**
	*	Mutator for m_Date - the Date of the chart. 
	*
	*	@param date is the boolean indicating whether to include a Date 
	*
	*	@return - success
	*/
	public boolean SetDate(Boolean date) {
		boolean success = false;
		if (date != null) {
			success = true;
			m_Date = date;
		}
		updateTitles();
		return success;
	}
	
	/**
	*	Accessor for m_Date
	*
	*	@return - Date Boolean
	*/
	public Boolean GetDate() {
		return m_Date;
	}	
	
	/**
	*	Mutator for m_Description - Description of the chart. 
	*
	*	@param descr is the String that the user has specified to be used
	*
	*	@return - success
	*/
	public boolean SetDescription(String descr){
		boolean success = false;
		if (descr != null) {
			success = true;
			m_Description = descr;
		}
		updateTitles();
		return success;
	}
	
	/**
	*	Accessor for m_Description
	*
	*	@return - Description String
	*/
	public String GetDescription() {
		return m_Description;
	}
	
	/**
	*	Mutator method for m_XAxis - X Axis Label of chart. 
	*
	*	@param xAxis is the String that the user has specified to be used
	*
	*	@return - success
	*/
	public boolean SetXAxis(String xAxis){
		
		//CANNOT PASS CHARTS WHICH ARENT OF THIS TYPE
		//RING,PIE,PIE3D, MULTIPLEPIECHART, MULTIPLEPIECHART3D
		boolean success = false;
		if (xAxis != null) {
			success = true;
			m_XAxis = xAxis;
		}
		updateTitles();		
		return success;
	}
	
	/**
	*	Accessor for m_XAxis
	*
	*	@return - X Axis String
	*/
	public String GetXAxis() {
		return m_XAxis;
	}
	
		/**
	*	Mutator method of m_YAxis - Y Axis Label of chart. 
	*
	*	@param - yAxis is the String that the user has specified to be used
	*
	*	@return - success
	*/
	public boolean SetYAxis(String yAxis){
		
		//CANNOT PASS CHARTS WHICH ARENT OF THIS TYPE
		//RING,PIE,PIE3D, MULTIPLEPIECHART, MULTIPLEPIECHART3D
		boolean success = false;
		if (yAxis != null) {
			success = true;
			m_YAxis = yAxis;
		}
		updateTitles();		
		return success;
	}
	
	/**
	*	Accessor for m_YAxis
	*
	*	@return - Y Axis String
	*/
	public String GetYAxis() {
		return m_YAxis;
	}
	
	/**
	*	Accessor for m_Chart - internal chart structure
	*	@return - memmory address of internal chart structure
	*/
	public JFreeChart GetChart() {
		return m_Chart;
	}
	
	/**
	*	Mutator for m_Chart - internal data storage
	*
	*	@param chart - chart data to be loaded into class
	*
	*	@return - success
	*/
	public boolean SetChart(JFreeChart chart) {
		boolean success = false;
		if (chart != null) {
			m_Chart = chart;
			success = true;
		}
		return success;
	}
	
	/**
	*	Mutator method for m_Colour - colour scheme of chart
	*
	*	@param colour - ColourList of colours to be applied
	*
	*	@return - success
	*/
	public boolean SetColour(ColourList colour) {
		boolean success = super.SetColour(colour);
		if (success) {
			ApplyColour(GetColour());
		}
		return success;
	}
	
	/**
	*	Mutator method of m_Legend - legent of chart
	*
	*	@param - legend is the LegendTitle that the user has 
	*	specified to be used
	*
	*	@return - success
	*/
	public boolean SetLegend(LegendTitle legend) {
		boolean success = false;
		if (legend != null) {
			success = true;
			m_Legend = legend;
		}
		updateTitles();		
		return success;
	}
	
	/**
	*	Accessor for m_Legend
	*	@return - LegendTitle Legend
	*/
	public LegendTitle GetLegend() {
		return m_Legend;
	}
	
	/**
	*	Internal method to update all subtitles in class
	*
	*	@return - success
	*/
	private boolean updateTitles() {
		GetChart().clearSubtitles();
		
		GetChart().setTitle(GetChartTitle());
		
		TextTitle	authorTitle = new TextTitle(GetAuthor());
		authorTitle.setPosition(RectangleEdge.TOP);
		authorTitle.setHorizontalAlignment(HorizontalAlignment.RIGHT);
		GetChart().addSubtitle(authorTitle);
		
		if(GetDate() == true){
		DateTitle date = new DateTitle(DateFormat.FULL);
		GetChart().addSubtitle(date);
		}
		
		if(GetLegend() != null){
		GetChart().addLegend(GetLegend());
		}
		
		TextTitle description = new TextTitle(GetDescription());
		description.setPosition(RectangleEdge.BOTTOM);
		GetChart().addSubtitle(description);
		
		String plottype = GetChart().getPlot().getPlotType();
		if (plottype.equals("Category Plot")) {
			CategoryPlot plot = (CategoryPlot) GetChart().getPlot();
			plot.getDomainAxis().setLabel(GetXAxis());
		} else if (plottype.equals("XY Plot")) {
			XYPlot plot = (XYPlot) GetChart().getPlot();
			plot.getDomainAxis().setLabel(GetXAxis());
		}
		
		plottype = GetChart().getPlot().getPlotType();
		if (plottype.equals("Category Plot")) {
			CategoryPlot plot = (CategoryPlot) GetChart().getPlot();
			plot.getRangeAxis().setLabel(GetYAxis());
		} else if(plottype.equals("XY Plot")) {
			XYPlot plot = (XYPlot) GetChart().getPlot();
			plot.getRangeAxis().setLabel(GetYAxis());
		}
		return true;
	}
	
	/**
	*	Abstract method to create a chart from data and title
	*
	*	@param data - graph data to build chart from
	*	@param title - title of the chart
	*
	*	@return - visual chart to be displayed on screen
	*/
	abstract JFreeChart CreateChart(Dataset data, String title);
	/**
	*	Abstract method to apply colour to the generated chart
	*
	*	@param colourScheme - list of colours that comprise the colour scheme
	*
	*	@return - success
	*/
	abstract boolean ApplyColour(ColourList colourScheme);
	/**
	*	Abstract method to create JFreeChart dataset from internal ChartData
	*
	*	@param data - data of type ChartData
	*
	*	@return - data of type Dataset
	*/
	abstract Dataset CreateDataset(ChartData data);
	
	/**
	*	Constructor from data, title and colourScheme
	*
	*	@param data - chart specific data
	*	@param title - title of the chart
	*	@param colourScheme - the colour scheme to draw the graph with
	*	@param userList - the user defined list to create the colours with
	*
	*	@return - success
	*/
	public Chart(ChartData data, String title, String colourScheme,
												ArrayList<Color> userList) {
		super();
		if (colourScheme == null) {
			colourScheme = ColourGenerator.RANDOM;
		}
		SetTitle(title);
		SetData(data);
		initChart(colourScheme, userList);
		SetLegend(GetChart().getLegend());
		SetChartTitle(title);
	}	
	
	/**
	*	Internal function to initualise chart
	*
	*	@param colourScheme - the colour scheme to draw the chart with
	*	@param userList - the user defined list to create the colours with
	*
	*/
	private void initChart(String colourScheme, ArrayList<Color> userList) {
		SetChart(CreateChart(CreateDataset(GetData()), GetTitle()));
		if (colourScheme.equals("C_USERDEFINED")){
			SetColour(new ColourList(userList));
		} else {
			SetColour(new ColourList(GetData().Width(), colourScheme));
		}
		ChartPanel	chartPanel = new ChartPanel(GetChart());
		add(chartPanel);
	}
}
