/**
* 	@file HistogramChart.java
* 	@author Connor Lynch
* 	@date 03 Apr 2013
* 
*	@see Chart, ColourGenerator, ColourList, ChartData
*
*	@brief Stores a user defined HistogramChart to be displayed on screen
*	
*	Provides an interface for designing and displaying a HistogramChart
*	Storing internal chart data and providing visual functions.
* 
*/



import	org.jfree.chart.ChartFactory;
import	org.jfree.chart.JFreeChart;
import  org.jfree.chart.plot.PlotOrientation;
import  org.jfree.data.general.Dataset;
import  org.jfree.data.statistics.HistogramDataset;
import  org.jfree.chart.plot.XYPlot;
import  org.jfree.data.statistics.HistogramType;
import  java.util.ArrayList;
import  java.awt.Color;

import  javax.swing.JFrame;



public class HistogramChart extends Chart {
	
	/**
	* Construct a HistogramChart
	*
	* @param data is the data that shall be used to create the chart
	* @param title is the title to be used upon creation of the chart
	* @param colourScheme is a string of the type of colour to use 
	* @param userList is the ArrayList<Color> of user defined colours
	* 
	*/
	public HistogramChart(ChartData data, String title, String colourScheme, 
													ArrayList<Color> userList){
		super(data, title, colourScheme, userList);
		SetXAxis(GetData().GetColumn(0));
		SetDate(true);
	}

	/**
	* Create a chart of type HistogramChart. 
	*
	* @param plotData is the data that shall be used to create the chart
	* @param title is the title to be used upon creation of the chart
	*
	* @return the newly created HistogramChart as chart 
	*/
	public JFreeChart CreateChart(Dataset plotData, String title) {
	    JFreeChart chart = ChartFactory.createHistogram(
		title,						//	Chart title
		GetData().GetColumn(0),		// 	Domain/x axis label
		null, 						// 	Range/y axis label 
		(HistogramDataset) plotData,//	Data
		PlotOrientation.VERTICAL,	//	Orientation
		false,				//	Include legend
		true,				//	Include tooltips
		false);				//	Do not include URLs
		
		return chart;
	}
	
	/**
	* Change the currently selected chart to have a colourScheme 
	* as selected by the user 
	*
	* @param colourScheme is the Colour Scheme which has been selected by the user
	* which is to be mapped onto the charts data.
	*
	* @return boolean indicating True if colourScheme was successfully implemented
	* or false otherwise.
	*/
	boolean ApplyColour(ColourList colourScheme) {
		boolean	success = false;
		XYPlot plot = (XYPlot) GetChart().getPlot();
		if (plot != null) {
			for (int row = 0; row < GetData().Width(); row++) {
				plot.getRenderer().setSeriesPaint(row, GetColour().Get(row));
			}
			success = true;
		}
		return success;
	}

	/**
	* Create a dataset of type HistogramDataset using data of type ChartData 
	*
	* @param data is the data that shall be used to create the chart
	*
	* @return the newly created dataset as result 
	*/
	Dataset CreateDataset(ChartData data) {
		HistogramDataset result = new HistogramDataset();
		double[] dataArray = new double[data.Width()];
			for (int row = 0; row <= dataArray.length-1; row++) {
				dataArray[row] =  Double.parseDouble(data.Get(0, row));
			} 
		result.addSeries("Series 1", dataArray, data.Width());
		result.setType(HistogramType.FREQUENCY);
	    return result;
	}
	
	/**
	*	Unit test
	*	@param args - command line arguments (not used)
	*/
	public static void main(String[] args) {
		final int			WINDOW_SIZE = 700;
		final String[][]	TEST_DATA = {{"5","14","3","25","32",
										"23","21","6","5","11"}};
		final String[]		TEST_COLUMNS = {"Test"};
		JFrame		frame = new JFrame("HistogramChart Test");
		ChartData 	data = new ChartData(TEST_COLUMNS, TEST_DATA);
		HistogramChart	chart = new HistogramChart(data, 
					"Histogram Chart Test", ColourGenerator.WARM,null);
		System.out.println(chart);
		frame.add(chart);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setSize(WINDOW_SIZE, WINDOW_SIZE);
		frame.setVisible(true);		
	}
	
	
} 