/**
* 	@file XYBarChart.java
* 	@author Connor Lynch
* 	@date 03 Apr 2013
* 
*	@see Chart, ColourGenerator, ColourList, ChartData
*
* 	@brief Header file for class which creates a XY Bar Chart
*
* 	Provides an interface for designing and displaying a StackedBarChart3D
*	Storing internal chart data and providing visual functions.
* 
*/



import org.jfree.chart.ChartFactory;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.data.xy.IntervalXYDataset;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;
import org.jfree.chart.renderer.xy.XYBarRenderer;
import org.jfree.data.general.Dataset;
import java.util.ArrayList;
import java.awt.Color;

import	javax.swing.JFrame;


public class XYBarChart extends Chart {
	
	/**
	* Construct a XY Bar Chart
	*
	* @param data is the data that shall be used to create the chart
	* @param title is the title to be used upon creation of the chart
	* @param colourScheme is a string of the type of colour to use 
	* @param userList is the ArrayList<Color> of user defined colours
	* 
	*/
	public XYBarChart(ChartData data, String title, String colourScheme, 
													ArrayList<Color> userList){
		super(data, title, colourScheme, userList);
		SetLegend(GetChart().getLegend());
		SetXAxis(data.GetColumn(0));
		//SetYAxis(data.GetColumn(1));
		SetDate(true);
	}

	/**
	* Create a chart of type XYBarChart 
	*
	* @param plotData is the data that shall be used to create the chart
	* @param title is the title to be used upon creation of the chart
	*
	* @return the newly created XY Bar Chart as chart 
	*/
	public JFreeChart CreateChart(Dataset plotData, String title) {	
	    JFreeChart chart = ChartFactory.createXYBarChart(
		title,							//  Chart title
		GetData().GetColumn(0),			//  Domain/x axis label
		false,							//  Domain Date axis
		null,							//  Range/y-axis label
		(IntervalXYDataset) plotData,	//  Data
		PlotOrientation.VERTICAL,		//  Orientation of chart
		true,				//  Include legend
		true,				//  Include tooltips
		false);				//  Do not include URLs
				
	    return chart;
	}
	
	/**
	* Change the currently selected chart to have a colourScheme 
	* as selected by the user 
	*
	* @param colourScheme is the Colour Scheme which has been selected by the user
	* which is to be mapped onto the charts data.
	*
	* @return boolean indicating True if colourScheme was successfully implemented
	* or false otherwise.
	*/
	boolean ApplyColour(ColourList colourScheme) {
		boolean	success = false;
		XYPlot plot = (XYPlot) GetChart().getPlot();
		XYBarRenderer barRenderer = (XYBarRenderer)plot.getRenderer();
		if (plot != null) {
			for (int i = 0; i < GetData().Height(); i++) {
				barRenderer.setSeriesPaint(i, GetColour().Get(i));
			}
			success = true;
		}
		return success;
	}
	
	/**
	* Create a dataset of type XYSeriesCollection using data of type ChartData 
	*
	* @param data is the data that shall be used to create the chart
	*
	* @return the newly created dataset as result 
	*/
	Dataset CreateDataset(ChartData data) {
		XYSeries series1;
		XYSeriesCollection result = new XYSeriesCollection();
		for (int c = 1; c < data.Height()	; c++){
			series1 = new XYSeries(GetData().GetColumn(c));
			series1.clear();
			for (int i = 0; i < data.Width()	; i++) {
				series1.add(Double.parseDouble(data.Get(0, i)), 
							Double.parseDouble(data.Get(c, i)));
			
			}
		result.addSeries(series1);
		}
	    return result;
	}
	
	/**
	*	Unit test
	*	@param args - command line arguments (not used)
	*/
	public static void main(String[] args) {
		final int			WINDOW_SIZE = 700;
		final String[][]	TEST_DATA = {{"5","14","3","25","32",
										"23","21","6","9","11"},
										{"23","21","6","5","11",
										"5","14","3","25","32"}};
		final String[]		TEST_COLUMNS = {"Test", "Data"};
		JFrame		frame = new JFrame("Scatter Test");
		ChartData 	data = new ChartData(TEST_COLUMNS, TEST_DATA);
		XYBarChart chart = new XYBarChart(data, "XY Bar Chart Test", 
												ColourGenerator.WARM,null);
		System.out.println(chart);
		frame.add(chart);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setSize(WINDOW_SIZE, WINDOW_SIZE);
		frame.setVisible(true);		
	}
} 